'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.ResizeHorizon = exports.ResizeVertical = exports.Resize = undefined;

var _react = require('react');

var React = _interopRequireWildcard(_react);

var _hammerjs = require('hammerjs');

var _hammerjs2 = _interopRequireDefault(_hammerjs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function createId() {
    var id = -1;
    return function () {
        id = id + 1;
        return id;
    };
}
var getResizeId = createId();

var Resize = exports.Resize = React.createClass({
    displayName: 'Resize',
    getResize: function getResize() {
        var $resize = document.querySelectorAll('.resize');
        var id = this.state.resizeId;
        var index = -1;
        for (var i = 0; i < $resize.length; i++) {
            if (id == $resize[i].getAttribute('data-resize-id')) {
                index = i;
            }
        }
        return $resize[index];
    },
    getResizeElement: function getResizeElement(className) {
        var $resize = this.getResize();
        if (className === 'resize') {
            return $resize;
        }

        var $child = $resize.childNodes;
        var list = [];
        for (var i = 0; i < $child.length; i++) {
            if ($child[i].classList.contains(className)) {
                list.push($child[i]);
            }
        }
        return list;
    },
    stopUserSelect: function stopUserSelect() {
        if (window.getSelection) {
            var selection = window.getSelection();
            selection.collapse(document.body, 0);
        }

        document.body.style.userSelect = 'none';
        document.body.style.WebkitUserSelect = 'none';
        document.body.style.MozUserSelect = 'none';
        document.body.style.MsUserSelect = 'none';
    },
    onUserSelect: function onUserSelect() {
        document.body.style.userSelect = '';
        document.body.style.WebkitUserSelect = '';
        document.body.style.MozUserSelect = '';
        document.body.style.MsUserSelect = '';
    },
    eventHandle: function eventHandle() {
        var _this = this;

        var type = this.state.resizeType;
        var $handle = [];
        if (type === 'vertical') {
            $handle = this.getResizeElement('resize-handle-vertical');
        } else if (type === 'horizon') {
            $handle = this.getResizeElement('resize-handle-horizon');
        }

        for (var i = 0; i < $handle.length; i++) {
            var hammertime = new _hammerjs2.default($handle[i]);
            hammertime.get('pan').set({ threshold: 1 });

            hammertime.on('panstart', function (ev) {
                _this.stopUserSelect();
                _this.state.onResizeStart(_this.getResizeInf());
            });
            hammertime.on('panend', function (ev) {
                _this.onUserSelect();
                _this.state.onResizeStop(_this.getResizeInf());
            });
            hammertime.on('panmove', function (ev) {
                if (type === 'vertical') {
                    _this.resizeVertical(ev);
                    _this.state.onResizeMove(_this.getResizeInf());
                } else if (type === 'horizon') {
                    _this.resizeHorizon(ev);
                    _this.state.onResizeMove(_this.getResizeInf());
                }
            });
        }
    },
    getResizeInf: function getResizeInf() {
        var type = this.state.resizeType;
        var childs = [];
        if (type === 'vertical') {
            var $vertical = this.getResizeElement('resize-vertical');
            for (var i = 0; i < $vertical.length; i++) {
                childs.push({ height: $vertical[i].offsetHeight });
            }
        } else if (type === 'horizon') {
            var $horizon = this.getResizeElement('resize-horizon');
            for (var i = 0; i < $horizon.length; i++) {
                childs.push({ width: $horizon[i].offsetWidth });
            }
        }

        return {
            type: type,
            resizeId: this.state.resizeId,
            resizeChilds: childs
        };
    },
    initialVertical: function initialVertical($resize, $vertical) {
        var handleHeight = parseInt(this.state.handleWidth);
        var handleColor = this.state.handleColor;
        var sumHeight = 0;

        for (var i = 0; i < $vertical.length; i++) {
            // vertical style
            var height = $vertical[i].getAttribute('data-height');
            var minHeight = $vertical[i].getAttribute('data-min-height');
            var overflow = $vertical[i].getAttribute('data-overflow');
            $vertical[i].style.overflow = overflow;
            $vertical[i].style.height = height + 'px';
            $vertical[i].style.position = 'relative';
            $vertical[i].setAttribute('min-height', minHeight);

            // handle style
            var $handle = document.createElement('div');
            $handle.className = 'resize-handle-vertical';
            $handle.style.height = handleHeight + 'px';
            $handle.style.cursor = 's-resize';
            $handle.style.backgroundColor = handleColor;

            if ($vertical[i + 1]) {
                $vertical[i].parentNode.insertBefore($handle, $vertical[i].nextSibling);
                sumHeight += handleHeight;
            }

            if ($vertical.length - 1 != i) {
                sumHeight += $vertical[i].offsetHeight;
            }
        }
        $vertical[$vertical.length - 1].style.height = $resize.offsetHeight - sumHeight + 'px';
    },
    resizeVertical: function resizeVertical(e) {
        if (e.velocityY === 0) return;
        var $resize = this.getResizeElement('resize');
        var $vertical = this.getResizeElement('resize-vertical');
        var $handle = this.getResizeElement('resize-handle-vertical');
        if (!$resize || $vertical.length === 0 || $handle.length === 0) return;

        var direction = e.velocityY > 0 ? 'down' : 'up';
        var $prev = e.target.previousElementSibling;
        var $next = e.target.nextElementSibling;
        var prevMinHeight = parseInt($prev.getAttribute('min-height'));
        var nextMinHeight = parseInt($next.getAttribute('min-height'));

        if (direction === 'up') {
            var indx_prev = $vertical.indexOf($prev);
            for (var i = $vertical.length - 1; 0 <= i; i--) {
                if (indx_prev >= i) {
                    if ($prev.offsetHeight <= prevMinHeight) {
                        $prev = $vertical[i];
                        prevMinHeight = parseInt($prev.getAttribute('min-height'));
                    }
                }
            }
        } else if (direction === 'down') {
            var indx_next = $vertical.indexOf($next);
            for (var i = 0; i < $vertical.length; i++) {
                if (indx_next <= i) {
                    if ($next.offsetHeight <= nextMinHeight) {
                        $next = $vertical[i];
                        nextMinHeight = parseInt($next.getAttribute('min-height'));
                    }
                }
            }
        }

        var prevHeight = 0;
        var nextHeight = 0;
        var sumPrevHeight = 0;
        var sumHeight = 0;
        var flag = true;
        for (var i = 0; i < $vertical.length; i++) {
            if (flag) {
                if ($handle[i] === e.target) {
                    flag = false;
                }
                if ($vertical[i] != $prev) {
                    sumPrevHeight += $vertical[i].offsetHeight;
                    sumPrevHeight += $handle[i].offsetHeight;
                }
            }

            if ($vertical[i] !== $prev && $vertical[i] !== $next) {
                sumHeight += $vertical[i].offsetHeight;
            }

            if ($handle[i]) {
                sumHeight += $handle[i].offsetHeight;
            }
        }
        prevHeight = e.center.y - sumPrevHeight - $resize.getBoundingClientRect().top;
        nextHeight = $resize.offsetHeight - (sumHeight + prevHeight);

        if (prevHeight < prevMinHeight) prevHeight = prevMinHeight;
        if (nextHeight < nextMinHeight) nextHeight = nextMinHeight;

        if (direction === 'down') {
            prevHeight = $resize.offsetHeight - sumHeight - nextHeight;
            if (prevHeight < prevMinHeight) prevHeight = prevMinHeight;
        } else if (direction === 'up') {
            nextHeight = $resize.offsetHeight - sumHeight - prevHeight;
            if (nextHeight < nextMinHeight) nextHeight = nextMinHeight;
        }

        if ($resize.offsetHeight === sumHeight + prevHeight + nextHeight) {
            $prev.style.height = prevHeight + 'px';
            $next.style.height = nextHeight + 'px';
        }
    },
    windowResizeVertical: function windowResizeVertical() {
        var $resize = this.getResizeElement('resize');
        var $vertical = this.getResizeElement('resize-vertical');
        var $handle = this.getResizeElement('resize-handle-vertical');
        if (!$resize || $vertical.length === 0 || $handle.length === 0) return;

        var sum = 0;
        var remain = 0;
        for (var i = 0; i < $vertical.length; i++) {
            sum += $vertical[i].offsetHeight;
        }
        for (var i = 0; i < $handle.length; i++) {
            sum += $handle[i].offsetHeight;
        }
        remain = $resize.offsetHeight - sum;

        if (remain > 0) {
            var last_idx = $vertical.length - 1;
            $vertical[last_idx].style.height = remain + $vertical[last_idx].offsetHeight + 'px';
        } else if (remain < 0) {
            for (var i = $vertical.length - 1; 0 <= i; i--) {
                var min_height = parseInt($vertical[i].getAttribute('min-height'));
                remain += $vertical[i].offsetHeight - min_height;

                if (remain >= 0) {
                    $vertical[i].style.height = remain + min_height + 'px';
                    break;
                }
                $vertical[i].style.height = min_height + 'px';
            }
        }

        this.state.onResizeWindow(this.getResizeInf());
    },
    initialHorizon: function initialHorizon($resize, $horizon) {
        var handleWidth = parseInt(this.state.handleWidth);
        var handleColor = this.state.handleColor;
        var sumWidth = 0;

        for (var i = 0; i < $horizon.length; i++) {
            // horizon style
            var width = $horizon[i].getAttribute('data-width');
            var minWidth = $horizon[i].getAttribute('data-min-width');
            var overflow = $horizon[i].getAttribute('data-overflow');
            $horizon[i].style.overflow = overflow;
            $horizon[i].style.width = width + 'px';
            $horizon[i].style.height = '100%';
            $horizon[i].style.float = 'left';
            $horizon[i].style.position = 'relative';
            $horizon[i].setAttribute('min-width', minWidth);

            // handle style
            var $handle = document.createElement('div');
            $handle.className = 'resize-handle-horizon';
            $handle.style.width = handleWidth + 'px';
            $handle.style.cursor = 'w-resize';
            $handle.style.backgroundColor = handleColor;
            $handle.style.height = '100%';
            $handle.style.float = 'left';

            if ($horizon[i + 1]) {
                $horizon[i].parentNode.insertBefore($handle, $horizon[i].nextSibling);
                sumWidth += handleWidth;
            }
            if ($horizon.length - 1 != i) {
                sumWidth += $horizon[i].offsetWidth;
            }
        }
        $horizon[$horizon.length - 1].style.width = $resize.offsetWidth - sumWidth + 'px';
    },
    resizeHorizon: function resizeHorizon(e) {
        if (e.velocityX === 0) return;
        var $resize = this.getResizeElement('resize');
        var $horizon = this.getResizeElement('resize-horizon');
        var $handle = this.getResizeElement('resize-handle-horizon');
        if (!$resize || $horizon.length === 0 || $handle.length === 0) return;

        var direction = e.velocityX > 0 ? 'right' : 'left';
        var $prev = e.target.previousElementSibling;
        var $next = e.target.nextElementSibling;
        var prevMinWidth = parseInt($prev.getAttribute('min-width'));
        var nextMinWidth = parseInt($next.getAttribute('min-width'));

        if (direction === 'left') {
            var indx_prev = $horizon.indexOf($prev);
            for (var i = $horizon.length - 1; 0 <= i; i--) {
                if (indx_prev >= i) {
                    if ($prev.offsetWidth <= prevMinWidth) {
                        $prev = $horizon[i];
                        prevMinWidth = parseInt($prev.getAttribute('min-width'));
                    }
                }
            }
        } else if (direction === 'right') {
            var indx_next = $horizon.indexOf($next);
            for (var i = 0; i < $horizon.length; i++) {
                if (indx_next <= i) {
                    if ($next.offsetWidth <= nextMinWidth) {
                        $next = $horizon[i];
                        nextMinWidth = parseInt($next.getAttribute('min-width'));
                    }
                }
            }
        }

        var prevWidth = 0;
        var nextWidth = 0;
        var sumPrevWidth = 0;
        var sumWidth = 0;
        var flag = true;
        for (var i = 0; i < $horizon.length; i++) {
            if (flag) {
                if ($handle[i] === e.target) {
                    flag = false;
                }
                if ($horizon[i] != $prev) {
                    sumPrevWidth += $horizon[i].offsetWidth;
                    sumPrevWidth += $handle[i].offsetWidth;
                }
            }

            if ($horizon[i] !== $prev && $horizon[i] !== $next) {
                sumWidth += $horizon[i].offsetWidth;
            }

            if ($handle[i]) {
                sumWidth += $handle[i].offsetWidth;
            }
        }
        prevWidth = e.center.x - sumPrevWidth - $resize.getBoundingClientRect().left;
        nextWidth = $resize.offsetWidth - (sumWidth + prevWidth);

        if (prevWidth < prevMinWidth) prevWidth = prevMinWidth;
        if (nextWidth < nextMinWidth) nextWidth = nextMinWidth;

        if (direction === 'right') {
            prevWidth = $resize.offsetWidth - sumWidth - nextWidth;
            if (prevWidth < prevMinWidth) prevWidth = prevMinWidth;
        } else if (direction === 'left') {
            nextWidth = $resize.offsetWidth - sumWidth - prevWidth;
            if (nextWidth < nextMinWidth) nextWidth = nextMinWidth;
        }

        if ($resize.offsetWidth === sumWidth + prevWidth + nextWidth) {
            $prev.style.width = prevWidth + 'px';
            $next.style.width = nextWidth + 'px';
        }
    },
    windowResizeHorizon: function windowResizeHorizon(e) {
        var $resize = this.getResizeElement('resize');
        var $horizon = this.getResizeElement('resize-horizon');
        var $handle = this.getResizeElement('resize-handle-horizon');
        if (!$resize || $horizon.length === 0 || $handle.length === 0) return;

        var sum = 0;
        var remain = 0;
        for (var i = 0; i < $horizon.length; i++) {
            sum += $horizon[i].offsetWidth;
        }
        for (var i = 0; i < $handle.length; i++) {
            sum += $handle[i].offsetWidth;
        }
        remain = $resize.offsetWidth - sum;

        if (remain > 0) {
            $horizon[$horizon.length - 1].style.width = remain + $horizon[$horizon.length - 1].offsetWidth + 'px';
        } else if (remain < 0) {
            for (var i = $horizon.length - 1; 0 <= i; i--) {
                var min_width = parseInt($horizon[i].getAttribute('min-width'));
                remain += $horizon[i].offsetWidth - min_width - 1;0;

                if (remain >= 0) {
                    $horizon[i].style.width = remain + min_width + 'px';
                    break;
                }
                $horizon[i].style.width = min_width + 'px';
            }
        }

        this.state.onResizeWindow(this.getResizeInf());
    },
    initialResize: function initialResize() {
        var type = this.state.resizeType;
        var $resize = this.getResizeElement('resize');
        var $vertical = this.getResizeElement('resize-vertical');
        var $horizon = this.getResizeElement('resize-horizon');
        if (!$resize || $vertical.length === 0 && $horizon.length === 0) return;

        if (type === 'vertical') {
            // resize style
            $resize.style.position = 'absolute';
            $resize.style.top = '0';
            $resize.style.bottom = '0';
            $resize.style.left = '0';
            $resize.style.right = '0';

            this.initialVertical($resize, $vertical);
            this.eventHandle();
        } else if (type === 'horizon') {
            // resize style
            $resize.style.position = 'absolute';
            $resize.style.top = '0';
            $resize.style.bottom = '0';
            $resize.style.left = '0';
            $resize.style.right = '0';
            $resize.style.overflow = 'hidden';

            this.initialHorizon($resize, $horizon);
            this.eventHandle();
        }
    },
    componentDidMount: function componentDidMount() {
        this.initialResize();
        var type = this.state.resizeType;
        if (type === 'vertical') {
            window.addEventListener('resize', this.windowResizeVertical);
        } else if (type === 'horizon') {
            window.addEventListener('resize', this.windowResizeHorizon);
        }
    },
    componentWillUnmount: function componentWillUnmount() {
        var type = this.state.resizeType;
        if (type === 'vertical') {
            window.removeEventListener('resize', this.windowResizeVertical);
        } else if (type === 'horizon') {
            window.removeEventListener('resize', this.windowResizeHorizon);
        }
    },
    getInitialState: function getInitialState() {
        var handleWidth = this.props.handleWidth ? this.props.handleWidth : '5px';
        var handleColor = this.props.handleColor ? this.props.handleColor : '#999';
        var onResizeStart = this.props.onResizeStart ? this.props.onResizeStart : function () {};
        var onResizeStop = this.props.onResizeStop ? this.props.onResizeStop : function () {};
        var onResizeMove = this.props.onResizeMove ? this.props.onResizeMove : function () {};
        var onResizeWindow = this.props.onResizeWindow ? this.props.onResizeWindow : function () {};

        var resizeType = '';
        if (this.props.children.length > 0) {
            if (this.props.children[0].type.displayName === 'ResizeHorizon') {
                resizeType = 'horizon';
            } else if (this.props.children[0].type.displayName === 'ResizeVertical') {
                resizeType = 'vertical';
            }
        }

        return {
            resizeType: resizeType,
            resizeId: getResizeId(),
            handleWidth: handleWidth,
            handleColor: handleColor,
            onResizeStart: onResizeStart,
            onResizeStop: onResizeStop,
            onResizeMove: onResizeMove,
            onResizeWindow: onResizeWindow
        };
    },
    render: function render() {
        var id = this.state.resizeId;
        return React.createElement(
            'div',
            { className: 'resize', 'data-resize-id': id },
            this.props.children
        );
    }
});

var ResizeVertical = exports.ResizeVertical = React.createClass({
    displayName: 'ResizeVertical',
    render: function render() {
        var id = this.props.id ? this.props.id : '';
        var className = this.props.className ? this.props.className : '';
        var height = this.props.height ? this.props.height : '0';
        var minHeight = this.props.minHeight ? this.props.minHeight : '0';
        var overflow = this.props.overflow ? this.props.overflow : 'hidden';

        return React.createElement(
            'div',
            { id: id,
                className: "resize-vertical " + className,
                'data-height': parseInt(height),
                'data-min-height': parseInt(minHeight),
                'data-overflow': overflow },
            this.props.children
        );
    }
});

var ResizeHorizon = exports.ResizeHorizon = React.createClass({
    displayName: 'ResizeHorizon',
    render: function render() {
        var id = this.props.id ? this.props.id : '';
        var className = this.props.className ? this.props.className : '';
        var width = this.props.width ? this.props.width : '0';
        var minWidth = this.props.minWidth ? this.props.minWidth : '0';
        var overflow = this.props.overflow ? this.props.overflow : 'hidden';

        return React.createElement(
            'div',
            { id: id,
                className: "resize-horizon " + className,
                'data-width': parseInt(width),
                'data-min-width': parseInt(minWidth),
                'data-overflow': overflow },
            this.props.children
        );
    }
});